///////////////////////////////////////////////////////////////////////////////
//
//    (C) Copyright 1995 - 2007 OSR Open Systems Resources, Inc.
//    All Rights Reserved
//
//    This sofware is supplied for instructional purposes only.
//
//    OSR Open Systems Resources, Inc. (OSR) expressly disclaims any warranty
//    for this software.  THIS SOFTWARE IS PROVIDED  "AS IS" WITHOUT WARRANTY
//    OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION,
//    THE IMPLIED WARRANTIES OF MECHANTABILITY OR FITNESS FOR A PARTICULAR
//    PURPOSE.  THE ENTIRE RISK ARISING FROM THE USE OF THIS SOFTWARE REMAINS
//    WITH YOU.  OSR's entire liability and your exclusive remedy shall not
//    exceed the price paid for this material.  In no event shall OSR or its
//    suppliers be liable for any damages whatsoever (including, without
//    limitation, damages for loss of business profit, business interruption,
//    loss of business information, or any other pecuniary loss) arising out
//    of the use or inability to use this software, even if OSR has been
//    advised of the possibility of such damages.  Because some states/
//    jurisdictions do not allow the exclusion or limitation of liability for
//    consequential or incidental damages, the above limitation may not apply
//    to you.
//
//    OSR Open Systems Resources, Inc.
//    105 Route 101A Suite 19
//    Amherst, NH 03031  (603) 595-6500 FAX: (603) 595-6503
//    email bugs to: bugs@osr.com
//
//
//    MODULE:
//
//        OSRMEMDRVTEST.CPP
//
//    ABSTRACT:
//
//      Win32 console mode program to fire up requests to the
//      OSRMEMDRV driver.   
//
//    AUTHOR(S):
//
//        OSR Open Systems Resources, Inc.
// 
//    REVISION:   
//
//
//      This application program demonstates how to share memory between
//      a user mode program and a driver.   There are 2 methods for doing
//      this and the program and associated driver demonstrate this.  The
//      application will use memory allocated by the driver and shared to
//      the application by default.  In order to get this shared address, 
//      IOCTL_OSR_GETMEM must be used.   The read and write functions will
//      work on this buffer, until a IOCTL_OSR_SETSHAREMEM function is used.
//      when this is done, the application will use the memory shared between
//      the driver and application set by the IOCTL_OSR_SETSHAREMEM request.
//      The only way to revert modes is to exit the application and restart
//      the application.
//
///////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <windows.h>
#include <winioctl.h>
#include <osrmemdrv_ioctl.h>

//
// Simple test application for the OSRMEMDRV driver project.
//

__cdecl main(ULONG argc, LPSTR *argv)
{
    HANDLE          DriverHandle;
    DWORD           code;
    ULONG           index;
    UCHAR           WriteBuffer[OSR_MEMORY_SIZE];
    UCHAR           ReadBuffer[OSR_MEMORY_SIZE];
    UCHAR           ShareBuffer[OSR_MEMORY_SIZE];
    OVERLAPPED      WriteOverlapped;
    OVERLAPPED      DeviceIoOverlapped;
    DWORD           WaitResult;
    DWORD           BytesRead, BytesWritten;
    DWORD           Function;
    BOOLEAN         bUsingShareMem = FALSE;   // TRUE if SETSHAREMEM IOCTL sent and successful
    PUCHAR          GetMemAddress = NULL;
    int             setValue = 0;

    //
    // Build the overlapped structures, creating a couple of
    // events to use in the process
    //
    WriteOverlapped.Offset     = 0; 
    WriteOverlapped.OffsetHigh = 0; 
    WriteOverlapped.hEvent     = CreateEvent(NULL, TRUE, FALSE, NULL);

    DeviceIoOverlapped.Offset     = 0; 
    DeviceIoOverlapped.OffsetHigh = 0; 
    DeviceIoOverlapped.hEvent     = CreateEvent(NULL, TRUE, FALSE, NULL);

    //
    // Init the write and read buffers with known data
    //
    memset(ReadBuffer, 0xAA, sizeof(ReadBuffer));

    memset(WriteBuffer, 0xEE, sizeof(WriteBuffer));

    //
    // Open the sample PCI device
    //
    DriverHandle = CreateFile("\\\\.\\OSRMEMDRV",  // Name of the NT "device" to open
                            GENERIC_READ|GENERIC_WRITE,  // Access rights requested
                            0,                           // Share access - NONE
                            0,                           // Security attributes - not used!
                            OPEN_EXISTING,               // Device must exist to open it.
                            FILE_FLAG_OVERLAPPED,        // Open for overlapped I/O
                            0);                          // extended attributes - not used!

    //
    // If this call fails, check to figure out what the error is and report it.
    //
    if (DriverHandle == INVALID_HANDLE_VALUE) {

        code = GetLastError();

        printf("CreateFile failed with error 0x%x\n", code);

        return(code);
    }

    //
    // Infinitely print out the list of choices, ask for input, process
    // the request
    //
    while(TRUE)  {

        printf ("\nOSRMEMDRV TEST -- Functions:\n\n");
        printf ("\t1. Check READ buffer\n");
        printf ("\t2. Issue a WRITE\n");
        printf ("\t3. Show Write Buffer\n");
        printf ("\t4. Send SETSHAREMEM IOCTL\n");
        printf ("\t5. Send GETMEM IOCTL\n");
        printf ("\n\t0. Exit\n");
        printf ("\n\tSelection: ");
        scanf ("%x", &Function);

        switch(Function)  {

        case 1: {
            //
            // Send a read
            //
            if(bUsingShareMem) {
                //
                // If the IOCTL_OSR_SETSHAREMEM has been sent, then the 
                // driver will be taking any writes that we issue and copying
                // the data into the shared memory (ShareBuffer), since that memory is our
                // memory, we just have to verify that.
                //
                printf("Printing first 32 bytes of READ buffer\n");

                for(index = 0; index < 32; index++)  {

                    printf("0x%0x,",ShareBuffer[index]);
                }

            } else if(GetMemAddress){
                //
                // If the IOCTL_OSR_SETSHAREMEM has not been sent, then we should
                // be using the memory that was preallocated for us an mapped into
                // our address space by the driver.   The VA of the memory has to
                // have already been retrieved by us vai the IOCTL_OSR_GETMEM request.
                // if not, we have an error.
                for(index = 0; index < 32; index++)  {

                    printf("0x%0x,",GetMemAddress[index]);
                }

            } else {
                printf("Read failed with No shared memory address gotten.");
            }
            }
            break;

        case 2: {
            if(!bUsingShareMem && !GetMemAddress) {
                printf("Write failed with No shared memory address gotten.");
                break;
            }
            setValue++;
            memset(WriteBuffer,setValue,sizeof(WriteBuffer));
            //
            // Send a write
            //
            if (!WriteFile(DriverHandle,
                            WriteBuffer,
                            sizeof(WriteBuffer),
                            &index, 
                            &WriteOverlapped)) {

                code = GetLastError();

                printf("WriteFile failed with error 0x%x\n", code);

                return(code);
            }
            printf("WRITE pattern = %x\n",setValue);

            }
            break;


        case 3:
            //
            // Show WRITE buffer
            //
            printf("Printing first 32 bytes of WRITE buffer\n");

            for(index = 0; index < 32; index++)  {

                printf("0x%0x,",WriteBuffer[index]);
            }

            break;

        case 4: {
            //
            // Test the IOCTL interface
            //
            USER_MEMORY_ADDRESS memAddr;
            if (!DeviceIoControl(DriverHandle,
                                (DWORD)IOCTL_OSR_SETSHAREMEM,
                                0,      // Ptr to InBuffer
                                0,      // Length of InBuffer
                                &ShareBuffer,      // Ptr to OutBuffer
                                sizeof(ShareBuffer),      // Length of OutBuffer
                                &index, // BytesReturned
                                &DeviceIoOverlapped) )    // Ptr to Overlapped structure
                                {


                code = GetLastError();

                if(code != ERROR_IO_PENDING)  {

                    printf("DeviceIoControl failed with error 0x%x\n", code);

                    return(code);
                } else {
                    bUsingShareMem = TRUE;
                }
            }
           
            printf("DeviceIoControl IOCTL_OSR_SETSHAREMEM worked OK sharing %p. \n",&ShareBuffer);
            }
            break;

        case 5: {
            //
            // Test the IOCTL interface
            //
            USER_MEMORY_ADDRESS memAddr;
            if (!DeviceIoControl(DriverHandle,
                                (DWORD)IOCTL_OSR_GETMEM,
                                0,      // Ptr to InBuffer
                                0,      // Length of InBuffer
                                &memAddr,      // Ptr to OutBuffer
                                sizeof(memAddr),      // Length of OutBuffer
                                &index, // BytesReturned
                                0) )    // Ptr to Overlapped structure
                                {

                code = GetLastError();

                printf("DeviceIoControl (reset) failed with error 0x%x\n", code);

                return(code);
            }
           
            printf("DeviceIoControl worked OK. %p\n",memAddr.UserMemoryAddress);
            GetMemAddress = (PUCHAR) memAddr.UserMemoryAddress;
            }
            break;

        case 0:

            //
            // zero is get out!
            //
            return(0);

        }
    }   
}
